#!/usr/bin/python
# -*- coding:utf-8 -*-

from .models import UserPerms,UserRolePerms
import sense_core as sd

def check_perm_by_uid(path,user_id):
    try:
        perm_id = UserPerms.find_perm_by_path(path)
        perm_list = UserRolePerms.find_permlist_by_uid(user_id)
        if perm_id in perm_list:
            return True
        else:
            return False
    except Exception as e:
        sd.log_exception(e)
        return False


import re
import json
import numpy
import pypinyin
import datetime
import base64


def chinese_to_pinyin(word, first=True):
    _res = []
    for i in pypinyin.pinyin(word, style=pypinyin.NORMAL):
        if first:
            _res.extend(i[0][0])
        else:
            _res.extend(i)
    return _res


def chinese_to_pinyin_str(word, first=True):
    return ''.join(chinese_to_pinyin(word, first))


def filter_html_tag(html, tag_name=''):
    if tag_name == '':
        return re.sub(r'</?\w+[^>]*>', '', html)
    return re.sub(r'</?' + tag_name + '>', '', html)


def format_zh_time(time):
    d1 = datetime.datetime.now()
    d2 = datetime.datetime.fromtimestamp(time)
    diff = int(round(d1.timestamp() - time))
    if diff < 60:
        return '刚刚'
    min = diff / 60
    if min < 60:
        return str(int(min)) + '分钟前'
    hour = diff / 3600
    if hour < 24 and d1.day == d2.day:
        return str(int(hour)) + '小时前'
    return d2.strftime('%Y-%m-%d')


def encode_token(*args):
    param = list(args)
    _data = str(param.pop(0))
    if len(param) == 0:
        raise ValueError('Encode_token:param must more than 2')
    for item in param:
        _data += ',' + str(item)
    token = base64.encodestring(_data.encode()).decode()
    return token


def decode_token(param):
    try:
        token = param.encode()
        data = base64.decodebytes(token)
        lis = data.decode().split(',')
        return lis
    except:
        return None


def str2int(param):
    try:
        item = int(param)
    except:
        item = None
    return item


class NumpyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        elif isinstance(obj, bytes):
            return str(obj)
        else:
            return super(NumpyEncoder, self).default(obj)


if __name__ == "__main__":
    t = datetime.datetime.now().timestamp() - 4500 * 12
    print(format_zh_time(t))



from django.db import models
import datetime
import re


class BaseModel(models.Model):
    id = models.AutoField(primary_key=True)

    class Meta:
        abstract = True

    @classmethod
    def find_map_by_ids(cls, ids):
        items = list(cls.objects.filter(id__in=ids))
        return {x.id: x for x in items}

    @classmethod
    def find_by_ids(cls, ids):
        map = cls.find_map_by_ids(ids)
        result = []
        for id in ids:
            if id in map:
                result.append(map[id])
        return result

    @classmethod
    def get_by_id(cls, id):
        try:
            return cls.objects.get(pk=id)
        except:
            return None

    @classmethod
    def find_many(cls, **kwargs):
        try:
            item = cls.objects.filter(**kwargs)
        except:
            item = None
        return item

    @classmethod
    def find_one_by(cls, **kwargs):
        try:
            item = cls.objects.get(**kwargs)
        except:
            item = None
        return item


class AuthUser(BaseModel):
    id = models.BigAutoField(primary_key=True)
    username = models.CharField(max_length=150)
    password = models.CharField(max_length=128)
    email = models.CharField(max_length=254)
    phone = models.CharField(max_length=64)
    last_name = models.CharField(max_length=30)
    first_name = models.CharField(max_length=30)
    is_staff = models.BooleanField()
    is_active = models.IntegerField()
    is_superuser = models.IntegerField()
    date_joined = models.DateTimeField()
    last_login = models.DateTimeField(blank=True)
    company = models.CharField(max_length=256)
    is_initial = models.IntegerField(default=1)

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'auth_user'


_groups_ = None
_last_time_ = datetime.datetime.now() + datetime.timedelta(minutes=-5)


class UserPermGroups(BaseModel):
    name = models.CharField(max_length=64)
    rule_prefix = models.CharField(max_length=64, unique=True)
    module = models.CharField(max_length=64, null=True)
    rank = models.IntegerField()
    create_time = models.DateTimeField(default=datetime.datetime.now())

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'user_perm_groups'

    @classmethod
    def match_request_path(cls, request_url, _now):
        for group in _groups_:
            if re.match(group.rule_prefix, request_url):
                return group.id
        return None

    @classmethod
    def update_groups(cls):
        global _groups_
        _groups_ = cls.objects.all().order_by('rank')


_perms_ = {}


class UserPerms(BaseModel):
    name = models.CharField(max_length=64)
    group_id = models.IntegerField(null=True)
    rule = models.CharField(max_length=64, unique=True)
    exclude_path = models.CharField(max_length=64, blank=True, null=True)
    rank = models.IntegerField()

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'user_perms'

    @classmethod
    def find_perm_by_path(cls, request_url):
        global _last_time_
        _now = datetime.datetime.now()
        if (_now - _last_time_).seconds >= 300:
            UserPerms.update_perms()
            UserPermGroups.update_groups()
            _last_time_ = _now
        group_id = UserPermGroups.match_request_path(request_url, _now)
        if not _perms_.__contains__(group_id):
            return None
        perms = _perms_[group_id]
        for perm in perms:
            if re.search(perm.rule, request_url):
                exclude_path_list = perm.exclude_path.split(',') if perm.exclude_path and perm.exclude_path != '' else []
                for exclude_path in exclude_path_list:
                    if re.search(exclude_path, request_url):
                        return None
                return perm.id
        return None

    @classmethod
    def update_perms(cls):
        global _perms_
        perms = cls.objects.all().order_by('rank')
        for item in perms:
            if not _perms_.__contains__(item.group_id):
                _perms_[item.group_id] = []
            _perms_[item.group_id].append(item)


class UserRoles(BaseModel):
    name = models.CharField(max_length=64)

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'user_roles'


class UserRoleRelations(BaseModel):
    uid = models.IntegerField()
    role_id = models.IntegerField()
    time = models.DateTimeField(default=datetime.datetime.now())

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'user_role_relations'

    @classmethod
    def find_role_by_uid(cls, user_id):
        role_list = cls.find_many(uid=user_id).values_list('role_id', flat=True)
        return role_list


class UserRolePerms(BaseModel):
    role_id = models.IntegerField()
    perm_id = models.IntegerField()

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'user_role_perms'

    @classmethod
    def find_permlist_by_uid(cls, uid):
        role_list = UserRoleRelations.find_role_by_uid(uid)
        perm_list = list(cls.objects.filter(role_id__in=role_list).values_list('perm_id', flat=True))
        return perm_list


class UserTokens(BaseModel):
    user_id = models.IntegerField()
    token = models.CharField(max_length=64, blank=True, null=True)
    last_token = models.CharField(max_length=64, blank=True, null=True)
    system = models.CharField(max_length=64, blank=True, null=True)
    login_time = models.DateTimeField()
    expired_time = models.DateTimeField(blank=True, null=True)

    class Meta:
        managed = False
        app_label = 'auth'
        db_table = 'user_tokens'



#!/usr/bin/python
# -*- coding:utf-8 -*-

from django.utils.deprecation import MiddlewareMixin
from django.contrib.auth.models import AnonymousUser
from .utils import decode_token, str2int
from .request_utils import RequestResult as Result
from .request_utils import parse_param, parse_int_param, get_request_uid, get_request_username, build_response
from .models import UserTokens, UserPerms, UserPermGroups, UserRoleRelations, UserRolePerms, AuthUser
import sense_core as sd
import datetime


class UserTokenCheckMiddleware(MiddlewareMixin):
    
    _sense_employee = int(sd.config('check_token', 'sensedeal', 5))
    _others = int(sd.config('check_token', 'others', 365))

    def _process_user_request(self, request):
        debug = sd.config('settings', 'debug', '0')
        if debug != '1':
            request.user = None
            return
        uid = parse_int_param(request, 'user_id')
        request.user = AuthUser.find_one_by(id=uid)

    def process_request(self, request):
        sd.log_init_config(root_path=sd.config('log_path'))
        try:
            self._process_user_request(request)
            if request.user and not isinstance(request.user, AnonymousUser):
                return None
            res = Result(Result.PLEASE_LOGIN)
            _token = request.META.get("HTTP_TOKEN")
            if not _token or len(_token) == 0:
                return None
            lis = decode_token(_token)
            if lis is None or len(lis) != 2:
                sd.log_info('lis:'+str(lis)+' , token is invalid')
                return build_response(res)
            user_token = UserTokens.find_one_by(token=lis[0])
            if not user_token:
                user_token = UserTokens.objects.filter(last_token=lis[0]).first()
                if user_token:
                    _now = datetime.datetime.now()
                    if not user_token.expired_time:
                        if user_token.system == 'sensedeal':
                            user_token.expired_time = _now + datetime.timedelta(days=self._sense_employee)
                        else:
                            user_token.expired_time = _now + datetime.timedelta(days=self._others)
                    diff = int(round(user_token.expired_time.timestamp() - _now.timestamp()))
                    #上一个token还未失效
                    if diff < 0:
                        sd.log_info('token is invalid ' + lis[0])
                        return build_response(res)
                else:
                    sd.log_info('token is invalid ' + lis[0])
                    return build_response(res)
            if user_token.system != lis[1]:
                sd.log_info('token system is invalid ' + lis[0] + ' system=' + list[1])
                return build_response(res)
            user = AuthUser.find_one_by(id=user_token.user_id)
            if user and hasattr(request, 'user'):
                request.user = user
                mutable = request.GET._mutable
                request.GET._mutable = True
                request.GET.__setitem__('user_id', user.id)
                request.GET._mutable = mutable
            return None
        except Exception as ex:
            sd.log_exception(ex)
            return build_response(Result(Result.SYSTEM_ERROR))


class PermissionCheckMiddleware(MiddlewareMixin):

    def process_request(self, request):
        try:
            res = Result(Result.NOT_BE_AUTHENTICATED)
            url_path = request.path
            perm_id = UserPerms.find_perm_by_path(url_path)
            if not perm_id:
                sd.log_info('pass perm for ' + url_path)
                return None
            if not hasattr(request, 'user') or request.user is None or isinstance(request.user, AnonymousUser):
                sd.log_info('request user is none')
                return build_response(res)
            user = request.user
            perm_list = UserRolePerms.find_permlist_by_uid(user.id)
            if perm_id in perm_list:
                return None
            else:
                sd.log_info('permission not in perm_list')
                return build_response(res)
        except Exception as ex:
            sd.log_exception(ex)
            return build_response(Result(Result.SYSTEM_ERROR))


class RequestLogMiddleware(MiddlewareMixin):

    def process_request(self, request):
        path = request.get_full_path().replace("[", "").replace("]", "")
        info = '[start][' + self.get_client_ip(request) + '][' + request.method + '][' + path + ']'
        sd.log_info(info)
        request.start_time = sd.get_current_millisecond()

    def process_response(self, request, response):
        cost = str(sd.get_current_millisecond() - request.start_time)
        path = request.get_full_path().replace("[", "").replace("]", "")
        info = '[end][' + self.get_client_ip(request) + '][' + request.method + '][' + str(
            get_request_uid(request)) + '][' + path + '][' + cost + ']'
        sd.log_info(info)
        return response

    def get_client_ip(self, request):
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip



import os
from setuptools import setup

requirements = [
    "sense_core>=0.1.4",
    "numpy>=1.15",
    "pypinyin>=0.33.1",
]

setup(
    name='sense-django',
    version='0.2.19',
    packages=[
            "sense_django",
    ],
    license='BSD License',  # example license
    description='sense django',
    install_requires=requirements,
    long_description=__doc__,
    url='',
    author='kafka0102',
    author_email='yujianjia@sensedeal.ai',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


from django.http import HttpResponse
import sense_core as sd
from .utils import NumpyEncoder
import json
import functools


class RequestResult(object):
    ERROR_PARAM_INVALID = 1
    ERROR_NOT_FOUND = 2
    PLEASE_LOGIN = 4
    NOT_BE_AUTHENTICATED = 5
    SYSTEM_ERROR = 6

    def get_error_message(self, code):
        if code == RequestResult.ERROR_PARAM_INVALID:
            return "参数错误"
        if code == RequestResult.ERROR_NOT_FOUND:
            return "数据无效"
        if code == RequestResult.PLEASE_LOGIN:
            return "会话失效，请重新登录"
        if code == RequestResult.NOT_BE_AUTHENTICATED:
            return "未授权用户，请联系管理员"
        if code == RequestResult.SYSTEM_ERROR:
            return "系统错误，请稍后重试"
        if code == 0:
            return ""
        return "未知错误"

    def set_error_code(self, code):
        self.set_code_message(code, self.get_error_message(code))
        return self

    def __init__(self, code=0, msg='', data=None):
        self.__dict_item = dict()
        self.__dict_item['ret_code'] = code
        if code != 0 and len(msg) == 0:
            msg = self.get_error_message(code)
        self.__dict_item['ret_msg'] = msg
        if data is not None:
            self.__dict_item['ret_data'] = data

    def set_error(self, message):
        self.set_code_message(-1, message)
        return self

    def set_code_message(self, code, message):
        self.__dict_item['ret_code'] = code
        self.__dict_item['ret_msg'] = message
        if code != 0 and 'ret_data' in self.__dict_item:
            del self.__dict_item['ret_data']
        return self

    def set_data(self, data):
        self.__dict_item['ret_data'] = data
        return self

    @classmethod
    def build_model_items(cls, items, type='list'):
        return sd.build_model_list(items, type)

    def add_data_item(self, key, val):
        if 'ret_data' not in self.__dict_item:
            self.__dict_item['ret_data'] = {}
        self.__dict_item['ret_data'][key] = val
        return self

    def package(self):
        return self.__dict_item


def build_response(result):
    return HttpResponse(json.dumps(result.package(), cls=NumpyEncoder), content_type="json")


def parse_param(request, key):
    if request.method == 'GET':
        return request.GET.get(key, '')
    if request.method == 'POST':
        return request.POST.get(key, '')
    return ''


def parse_pn_param(request):
    pn = parse_int_param(request, 'pn')
    return pn if pn >= 0 else 0


def parse_int_param(request, key, default=0):
    if request.method == 'GET':
        val = request.GET.get(key, '')
    elif request.method == 'POST':
        val = request.POST.get(key, '')
    else:
        val = ''
    if len(val) == 0:
        return default
    return int(val)


def get_request_uid(request):
    try:
        if request.user is None:
            return 0
        return request.user.id
    except:
        return 0


def get_request_username(request):
    if request.user is None:
        return ''
    return request.user.username


def catch_view_exception(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if sd.is_debug():
            return func(*args, **kwargs)
        try:
            return func(*args, **kwargs)
        except Exception as ex:
            sd.log_exception(ex)
            result = RequestResult(RequestResult.SYSTEM_ERROR)
            return build_response(result)

    return wrapper
